/* HelloCube.c

	This is a source code example of a program using "3D Game Machine"...
	*** this demo needs only 450 Kb to run ! ***
	
	You need to put your serial name and number in the gm_Serial() function before
	compiling.

	This program uses a source file called "MacOSUtils.c". It has been included to
	make sure the project will compile but it uses some _shameful_ routines, so
	avoid using it in a commercial application...
	
	3D Game Machine Copyright (c) Virtually Unlimited, SNC 1993-1995 all right reserved
*/

// system includes
#include "Quickdraw.h"
#include "Palettes.h"
#include "QDOffscreen.h"
#include "Gestalt.h"

// 3DGM includes

#include "gm_Types.h"
#include "gm_Core.h"
#include "gm_Err.h"

// System utilities

#include "MacOSUtils.h"

// some defines

#if (defined(powerc))
#define	Interlace	0
#else
#define	Interlace	1		// if compiled for 68K, uses 1/2 lines do go faster
#endif

#define	WorkBuffSize	60*1024L
#define	PicSize		128L

// global variables

Ptr		DaPicPtr;

// White light coefficients
static	gm_RGBA	WhiteAmbient[]		= {0xF000,0xF000,0xF000,0} ;
static	gm_RGBA	WhiteDiffuse[]		= {0xF000,0xF000,0xF000,0} ;
static	gm_RGBA	WhiteSpecular[]		= {0,0,0,0} ;

// sample material coefficients
static	gm_RGBA	Mat1Emission[]		= {0,0,0,0} ;
static	gm_RGBA	Mat1Ambient[]		= {0xC000,0xC000,0xC000,0} ;
static	gm_RGBA	Mat1Diffuse[]		= {0xC000,0xC000,0xC000,0} ;
static	gm_RGBA	Mat1Specular[]		= {0x2000,0x2000,0x2000,0} ;

static	gm_RGBA	Mat2Emission[]		= {0,0,0,0} ;
static	gm_RGBA	Mat2Ambient[]		= {0xFFFF,0x2000,0x2000,0} ;
static	gm_RGBA	Mat2Diffuse[]		= {0xFFFF,0x2000,0x2000,0} ;
static	gm_RGBA	Mat2Specular[]		= {0xFFFF,0xFFFF,0xFFFF,0} ;

static	short	CubeDots[]={
	
	#define	CubeSize	1000
	
	-CubeSize,-CubeSize,-CubeSize,
	-CubeSize,CubeSize,-CubeSize,
	CubeSize,CubeSize,-CubeSize,
	CubeSize,-CubeSize,-CubeSize,
	
	-CubeSize,-CubeSize,CubeSize,
	-CubeSize,CubeSize,CubeSize,
	CubeSize,CubeSize,CubeSize,
	CubeSize,-CubeSize,CubeSize,
	
	gm_EOD
	
	} ;

#define	MaterialID	1
#define	TextureID	0

static	short	CubePrimitives[]={
	
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,4,0,7,0,3,7,4,gm_EOP,		// this line is one polygon
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,7,6,3,3,2,6,7,gm_EOP,
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,5,4,1,4,5,1,0,gm_EOP,
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,5,6,4,7,6,5,4,gm_EOP,
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,6,5,2,2,1,5,6,gm_EOP,
	gm_TexPoly,0,MaterialID,TextureID,0,0,0,0,1,2,0,0,1,2,3,gm_EOP,
	
	gm_EOD
	
	} ;
	
static	short	TetraDots[]={
	
	-200,-100,0,
	200,-100,0,
	0,200,0,
	0,0,200,
	
	gm_EOD
	
	} ;

static	short	TetraPrims[]={
	
	gm_ShadedPoly,0,2,2,1,0,gm_EOP,		// bottom
	gm_ShadedPoly,0,2,1,2,3,gm_EOP,		// right
	gm_ShadedPoly,0,2,3,2,0,gm_EOP,		// left
	gm_ShadedPoly,0,2,0,1,3,gm_EOP,		// front
	
	gm_EOD
	
	} ;
	
//---- Quit ------------------------------------------

static
void	Quit(Str255 ErrStr)

{	if (ErrStr!=nil)	PrintError(ErrStr);
	
	// release Picture memory
	DisposePtr(DaPicPtr);
	
	// flush any keydown, mousedown, etc. events
	FlushEvents(everyEvent,0);
	
	// flush error variable
	gm_Flush3DGMError();
	
	// restore saved screen depth
	RestoreScreenDepth();
	// (doesn't check error)
	
	// exit 3DGM and release all memory
	gm_Exit3DGM();
	// (doesn't check error)
	
	ExitToShell();
}

//---- Set 3DGM's memory preferences ---------------------

static
void	Set3DGMPreferences(void)
	
	// change 3DGM's preferences to use less memory
	
{	gm_3DGMPrefs	DaPrefs;
	
	gm_Get3DGMPrefs(&DaPrefs);
	if (gm_Err)	Quit("\pgm_GetPrefs error");
	
	DaPrefs.primitiveBufferSize=1024L;
	DaPrefs.maxShapeNbr=5;
	DaPrefs.maxShapeDotNbr=200;
	DaPrefs.maxPolyDotNbr=10;
	DaPrefs.maxPolyNbr=20;
	DaPrefs.maxShapePolyNbr=20;
	DaPrefs.maxMaterials=4;
	DaPrefs.maxTexturePolyNbr=10;
	DaPrefs.maxTextures=4;
	DaPrefs.maxViewPortWidth=512;
	DaPrefs.maxViewPortHeight=512;
	DaPrefs.collisionBufferSize=0;
	
	gm_Set3DGMPrefs(&DaPrefs);
	if (gm_Err)	Quit("\pgm_SetPrefs error");
}

//---- MAIN CODE -----------------------------------------

void	main(void)

{	Ptr		DaWorkBuffer;
	
	gm_ColorPort	DaColorPort;
	gm_ViewPort	DaViewPort;
	gm_World	DaWorld;
	gm_Shape	TextureCube,Tetrahedron;
	gm_LightSource	WhiteLight;
	
	osu_ScreenParam	DaScreen;
	
	Handle		PltHdl;
	Ptr		TexPtr;
	short		DaTextureID;
	
	Boolean		TextureFlag=true;
	
	InitMac();		// (MacOS utilies call)
	HideCursor();
	
	SetScreenDepth(8,1);	// (MacOS utilies call)
	if (gm_Err)	Quit("\pCouldn't set screen depth");	
	
	#if !(defined(powerc))
	{	long	GesResp;
		short	err;
		
		err=Gestalt(gestaltFPUType,&GesResp);
		if (err)	Quit("\pGestalt error");
		if (GesResp==0)	TextureFlag=false;	// if no FPU then turn off textures
	}
	#endif
	
// INITIALIZE 3DGM
	
	gm_Serial( [ insert your serial number here ] );
	if (gm_Err)	Quit("\pgm_Serial number error");
	
	Set3DGMPreferences();		// subroutine defined above
	
	gm_Init3DGM();
	if (gm_Err)	Quit("\pgm_Init3DGM error");
	
	// allocate memory for ViewPort's work buffer
	
	DaWorkBuffer=NewPtr(WorkBuffSize);
	if (DaWorkBuffer==nil)	Quit("\pOut of Memory");
	
// COLOR PORT
	
	gm_InitColorPort(&DaColorPort);
	if (gm_Err)	Quit("\pgm_InitColorPort error");
	
	DaColorPort.colorMode=gm_12bitRamp;	// color fitting algorithm is a 12 bit color Ramp

	gm_NewColorPort(&DaColorPort);
	if (gm_Err)	Quit("\pgm_NewColorPort error");
	
	// (Load system palette)
	
	PltHdl=GetResource('clut',999);
	if (PltHdl==nil)	Quit("\pCLUT resource not found");
	HLock(PltHdl);
	gm_SetColorPortPalette(&DaColorPort,gm_CluttoRGBA(*PltHdl));	// macro for converting a CLUT to gm_RGBA* (defined in "MacOSUtils.h")
	if (gm_Err)		Quit("\pgm_SetColorPortPalette error");
	HUnlock(PltHdl);
	ReleaseResource(PltHdl);
	
	// update ColorPort since palette has changed,
	// you would normally use your own palette to get
	// nicer shading...
	gm_UpdateColorPort(&DaColorPort);
	if (gm_Err)	Quit("\pgm_UpdateColorPort error");
	
// WORLD
	
	gm_InitWorld(&DaWorld);
	if (gm_Err)	Quit("\pgm_InitWorld error");
	
	DaWorld.ambientPercentage=20;			// 20 % global ambient light
	DaWorld.ambientColor.R=65535;			// ambient color is white
	DaWorld.ambientColor.G=65535;
	DaWorld.ambientColor.B=65535;
	
	gm_NewWorld(&DaWorld);
	if (gm_Err)	Quit("\pgm_NewWorld error");
	
// VIEW PORT
	
	gm_InitViewPort(&DaViewPort);
	if (gm_Err)	Quit("\pgm_InitViewPort error");
	
	DaViewPort.world=&DaWorld;			// install World
	
	// get direct pointer to screen memory (a good boy wouldn't do this normally)
	
	GetScreenParams(&DaScreen);
	// position imageBuffer pointer to (64,50), 64 in width so it is alligned to a longword
	DaViewPort.imageBuffer=DaScreen.bitmap+(DaScreen.rowBytes*50)+64L;
	DaViewPort.width=400;
	DaViewPort.height=300;
	DaViewPort.rowBytes=DaScreen.rowBytes;
	
	#if	Interlace
	DaViewPort.height/=2;		// when interlacing, reduce size by 2
	DaViewPort.rowBytes*=2;		// increase rowBytes by two
	DaViewPort.vScale=50;		// and do a vertical scale of 50%
	#endif
	
	DaViewPort.workBuffer=DaWorkBuffer;		// install pointer to work memory buffer
	DaViewPort.workBufferSize=WorkBuffSize;		// and give work buffer's size
	DaViewPort.colorPort=&DaColorPort;		// pointer to current ColorPort
	
	// sets position and orientation of observer/camera
	
	DaViewPort.pos.x=0;
	DaViewPort.pos.y=-6000;
	DaViewPort.pos.z=0;
	DaViewPort.pos.alpha=0;
	DaViewPort.pos.beta=0;
	DaViewPort.pos.teta=0;
	
	DaViewPort.zoom=500;				// sets camera's opening angle
	DaViewPort.hither=100;				// sets front clipping plane (called "hither")
	DaViewPort.textureFlag=TextureFlag;		// set texture on/off flag
	
	gm_NewViewPort(&DaViewPort);
	if (gm_Err)	Quit("\pgm_NewViewPort error");
	
	#if	Interlace
	// (MacOS utilities call)
	ClrBox(DaViewPort.imageBuffer,DaViewPort.width,DaViewPort.height*2,DaViewPort.rowBytes/2,0xFF);
	#endif
	
	gm_ClearViewPort(&DaViewPort);		// clear the view port
	if (gm_Err)	Quit("\pgm_ClearViewPort error");

// MATERIAL
	
	gm_NewMaterial(1);	// ask for material in slot 1
	if (gm_Err)	Quit("\pgm_NewMaterial Error");
	
	// define this material's coefficients
	gm_DefineMaterial(1,Mat1Emission,Mat1Ambient,Mat1Diffuse,Mat1Specular,0);
	if (gm_Err)	Quit("\pgm_DefineMaterial Error");
	
	gm_NewMaterial(2);	// ask for material in slot 2
	if (gm_Err)	Quit("\pgm_NewMaterialID Error");
	
	// define this material's coefficients
	gm_DefineMaterial(2,Mat2Emission,Mat2Ambient,Mat2Diffuse,Mat2Specular,0);
	if (gm_Err)	Quit("\pgm_DefineMaterial Error");
	
// TEXTURE
	
	DaTextureID=gm_GetNewTexture();		// ask for any free texture slot
	if (gm_Err)	Quit("\pgm_GetNewTexture Error");
	
	TexPtr=LoadTexturePict(128,PicSize);	// gets pointer to picture resource 128 (MacOSUtils call)
	if (TexPtr==nil)	Quit("\pLoadTexturePict error");	// can't find res or out of memory error
	
	gm_DefineTexture(DaTextureID,TexPtr,PicSize,PicSize);	// define texture from loaded pict (width and height are the same)
	if (gm_Err)	Quit("\pgm_DefineTexture Error");
	
// TEXTURE CUBE SHAPE
	
	gm_InitShape(&TextureCube);
	if (gm_Err)	Quit("\pgm_InitShape error");
	
	TextureCube.dotsPtr=CubeDots;			// install pointer to shape dots
	TextureCube.primsPtr=CubePrimitives;		// install pointer to shape primitives
	
	// shape position and orientation
	TextureCube.pos.x=0;
	TextureCube.pos.y=3200;
	TextureCube.pos.z=0;
	TextureCube.pos.alpha=0;
	TextureCube.pos.beta=0;
	TextureCube.pos.teta=0;
	
	gm_NewShape(&TextureCube);
	if (gm_Err)	Quit("\pgm_NewShape error");
	
	gm_AttachShapeToWorld(&DaWorld,&TextureCube);	// atach shape to world
	if (gm_Err)	Quit("\pgm_AttachShapeToWorld error");
	
// TETRAHEDRON SHAPE
	
	gm_InitShape(&Tetrahedron);			// same as with shape 1
	if (gm_Err)	Quit("\pgm_InitShape error");
	
	Tetrahedron.dotsPtr=TetraDots;
	Tetrahedron.primsPtr=TetraPrims;
	
	// set Tetrahedron position and orientation
	Tetrahedron.pos.x=1000;
	Tetrahedron.pos.y=-2000;
	Tetrahedron.pos.z=800;
	Tetrahedron.pos.alpha=0;
	Tetrahedron.pos.beta=0;
	Tetrahedron.pos.teta=0;
	
	gm_NewShape(&Tetrahedron);
	if (gm_Err)	Quit("\pgm_NewShape error");
	
	gm_AttachShapeToWorld(&DaWorld,&Tetrahedron);	// attach shape 2 to world
	if (gm_Err)	Quit("\pgm_AttachShapeToWorld error");
	
// LIGHT SOURCE
	
	gm_InitLightSource(&WhiteLight);
	if (gm_Err)	Quit("\pgm_InitLightSource error");
	
	WhiteLight.type=gm_PositionalLight;	// lightsource type is positional
	
	// lightsource position
	WhiteLight.pos.x=-50000;
	WhiteLight.pos.y=-16000;
	WhiteLight.pos.z=30000;
	
	// lightsource coefficients
	WhiteLight.ambient=*WhiteAmbient;
	WhiteLight.diffuse=*WhiteDiffuse;
	WhiteLight.specular=*WhiteSpecular;
	
	gm_NewLightSource(&WhiteLight);
	if (gm_Err)	Quit("\pgm_NewLightSource error");
	
	gm_AttachLightSource(&DaWorld,&WhiteLight);	// attach light source to world
	if (gm_Err)	Quit("\pgm_AttachLightSource error");
	
	
// MAIN LOOP
	
	while (!Button())
	
	{	gm_RenderViewPort(&DaViewPort);		// render ViewPort into image buffer
		if (gm_Err)	Quit("\pgm_RenderViewPort error");
		
		// rotate Cube by 1 degree in alpha,beta and teta
		TextureCube.pos.alpha+=gm_Deg(1);
		TextureCube.pos.beta+=gm_Deg(1);
		TextureCube.pos.teta+=gm_Deg(1);
	
		// rotate Tetrahedron by 2 degrees in alpha and -2 degrees in beta
		Tetrahedron.pos.alpha+=gm_Deg(2);
		Tetrahedron.pos.beta-=gm_Deg(2);
	}
	
	Quit(nil);
	
}


// nada mas